﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using ThinkGeo.MapSuite.Core;

namespace TrackZoomTool
{
    public partial class Form1 : Form
    {
        private MapEngine mapEngine;
        private Bitmap bitmap;

        private bool isTrackZoom = false;
        private Point mouseStart;
        private Point mouseCurrent;
        private bool isTracking = false;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            bitmap = new Bitmap(MapContainer.Width, MapContainer.Height);

            mapEngine = new MapEngine();
            mapEngine.CurrentExtent = ExtentHelper.GetDrawingExtent(new RectangleShape(-163.4, 41.9, -60.1, 30.3), MapContainer.Width, MapContainer.Height);
            mapEngine.BackgroundFillBrush = new GeoSolidBrush(GeoColor.GeographicColors.ShallowOcean);

            ShapeFileFeatureLayer worldMapKitLayer = new ShapeFileFeatureLayer(@"..\..\Data\USStates.shp");
            worldMapKitLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.Country1;
            worldMapKitLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            mapEngine.StaticLayers.Add("WorldMapKitLayer", worldMapKitLayer);

            DrawImage();
        }

        private void DrawImage()
        {
            mapEngine.OpenAllLayers();
            mapEngine.DrawStaticLayers(bitmap, GeographyUnit.DecimalDegree);
            mapEngine.CloseAllLayers();

            MapContainer.Image = bitmap;
        }

        private void btnStartTrack_Click(object sender, EventArgs e)
        {
            isTrackZoom = true;
        }

        private void btnStopTrack_Click(object sender, EventArgs e)
        {
            isTrackZoom = false;
        }

        private void MapContainer_MouseDown(object sender, MouseEventArgs e)
        {
            if (isTrackZoom && e.Button == MouseButtons.Left)
            {
                mouseStart = e.Location;
                isTracking = true;
            }
        }

        private void MapContainer_MouseMove(object sender, MouseEventArgs e)
        {
            if (isTrackZoom && e.Button == MouseButtons.Left)
            {
                mouseCurrent = e.Location;
                MapContainer.Invalidate();
            }
        }

        private void MapContainer_Paint(object sender, PaintEventArgs e)
        {
            if (isTrackZoom && isTracking)
            {
                Graphics graphics = e.Graphics;
                int startX = Math.Min(mouseStart.X, mouseCurrent.X);
                int startY = Math.Min(mouseStart.Y, mouseCurrent.Y);
                int endX = Math.Max(mouseStart.X, mouseCurrent.X);
                int endY = Math.Max(mouseStart.Y, mouseCurrent.Y);

                graphics.DrawRectangle(new Pen(Color.Black, 2), new Rectangle(startX, startY, endX - startX, endY - startY));
                graphics.FillRectangle(new SolidBrush(Color.FromArgb(80, 239, 251, 239)), new Rectangle(startX, startY, endX - startX, endY - startY));
            }
        }

        private void MapContainer_MouseUp(object sender, MouseEventArgs e)
        {
            if (isTrackZoom)
            {
                isTracking = false;
                if (mouseStart.X != e.X && mouseStart.Y != e.Y)
                {
                    double startX = Math.Min(mouseStart.X, e.X);
                    double startY = Math.Min(mouseStart.Y, e.Y);
                    double endX = Math.Max(mouseStart.X, e.X);
                    double endY = Math.Max(mouseStart.Y, e.Y);

                    double widthFactor = mapEngine.CurrentExtent.Width / MapContainer.Width;
                    double heightFactor = mapEngine.CurrentExtent.Height / MapContainer.Height;

                    double upperLeftX = +(startX * widthFactor + mapEngine.CurrentExtent.UpperLeftPoint.X);
                    double upperLeftY = -(startY * heightFactor - mapEngine.CurrentExtent.UpperLeftPoint.Y);
                    double lowerRightX = +(endX * widthFactor + mapEngine.CurrentExtent.UpperLeftPoint.X);
                    double lowerRightY = -(endY * heightFactor - mapEngine.CurrentExtent.UpperLeftPoint.Y);
                    RectangleShape currentExtent = new RectangleShape(upperLeftX, upperLeftY, lowerRightX, lowerRightY);

                    mapEngine.CurrentExtent = ExtentHelper.GetDrawingExtent(currentExtent, MapContainer.Width, MapContainer.Height);
                    

                    DrawImage();
                    MapContainer.Invalidate();
                }
            }
            mouseStart = new Point();
        }

        private void btnFullExtent_Click(object sender, EventArgs e)
        {
            mapEngine.CurrentExtent = ExtentHelper.GetDrawingExtent(new RectangleShape(-163.4, 41.9, -60.1, 30.3), MapContainer.Width, MapContainer.Height);
            DrawImage();
        }
    }
}
